/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.activiti.timer;

import org.activiti.bpmn.model.TimerEventDefinition;
import org.activiti.engine.delegate.event.ActivitiEvent;
import org.activiti.engine.impl.persistence.entity.JobEntity;
import org.activiti.engine.runtime.ProcessInstance;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

@Component
public class FiredTimerRetryListener extends AbstractTimerRetryListener<JobEntity> {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    
    @Override
    public void onTimerAttempt(ActivitiEvent event, JobEntity job, TimerEventDefinition timerDef, int attempts) {
        this.logger.trace("onTimerAttempt({}, {}, {}, {})", event.getExecutionId(), job.getId(), timerDef.getId(), attempts);

        /**
         * If the timer is fired in the same transaction where the prcoess instance ends, SQL issue occur.
         * This is undoubtedly an Activiti bug, but we will do a check here just to avoid it.
         */
        ProcessInstance pi = this.services.getRuntimeService().createProcessInstanceQuery()
                .processInstanceId(job.getProcessInstanceId())
                .singleResult();
        if (pi == null) {
            this.logger.debug("The process instance could not be found; unable to increment the timer variable: {}", timerDef.getId());
        } else if (pi.isEnded()) {
            this.logger.debug("The process instance has ended; unable to increment the timer variable: {}", timerDef.getId());
        } else {
            attempts++;
            this.setAttempts(job, timerDef.getId(), attempts);
            this.logger.debug("Fired timer incremented attempts: {}: {}: {}", event.getProcessInstanceId(), timerDef.getId(), attempts);
        }
    }

}
