/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.activiti.timer;

import org.activiti.bpmn.model.BoundaryEvent;
import org.activiti.bpmn.model.Event;
import org.activiti.bpmn.model.TimerEventDefinition;
import org.activiti.engine.delegate.event.ActivitiEvent;
import org.activiti.engine.impl.persistence.entity.TimerJobEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

@Component
public class ScheduledTimerRetryListener extends AbstractTimerRetryListener<TimerJobEntity> {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Value("${inteligr8.timerExtension.recordAttemptOnScheduling:true}")
    private boolean recordAttemptOnScheduling;
    
    @Override
    public void onTimerExhaustion(ActivitiEvent event, Event modelEvent, TimerJobEntity job) {
        this.logger.trace("onTimerExhaustion({}, {}, {})", event.getExecutionId(), modelEvent.getId(), job.getId());
        
        if (event instanceof BoundaryEvent) {
            this.logger.info("Scheduled timer cancelled; will not retry any more: {}", event.getProcessInstanceId());
            this.services.getManagementService().deleteTimerJob(job.getId());
        } else {
            this.logger.info("Scheduled timer cancelled and moved to deadletter; will not automatically retry any more: {}", event.getProcessInstanceId());
            this.services.getManagementService().moveJobToDeadLetterJob(job.getId());
        }
    }
    
    @Override
    public void onTimerAttempt(ActivitiEvent event, TimerJobEntity job, TimerEventDefinition timerDef, int attempts) {
        this.logger.trace("onTimerAttempt({}, {}, {}, {})", event.getExecutionId(), job.getId(), timerDef.getId(), attempts);
        
        if (this.recordAttemptOnScheduling) {
            attempts++;
            this.setAttempts(job, timerDef.getId(), attempts);
            this.logger.debug("Scheduled timer incremented attempts: {}: {}: {}", event.getProcessInstanceId(), timerDef.getId(), attempts);
        }
    }

}
